defmodule Eakins.UseCases.DeleteSchemaImage do
  @moduledoc """
  Updates one of the images in a schema's image map
  """

  alias Ecto.{Changeset, Multi}

  alias Eakins.{Images, Repo, Schema}
  alias Eakins.UseCases.Reflection

  require Logger
  import Eakins.Gettext

  @spec call(module(), module() | String.t(), any(), String.t(), Images.key()) ::
          {:ok, Schema.t()} | {:error, Changeset.t()}

  def call(repo_module, schema_module, id, container_name, image_key)
      when is_binary(schema_module) do
    schema_module =
      ("Elixir." <> schema_module)
      |> String.to_existing_atom()

    call(repo_module, schema_module, id, container_name, image_key)
  end

  def call(repo_module, schema_module, id, container_name, image_key) do
    record = repo_module.get(id)

    changeset = Changeset.change(record)
    old_image = Reflection.find_image(record, container_name, image_key)

    Multi.new()
    |> Multi.update(
      :update_schema,
      Reflection.delete_image(record, container_name, changeset, image_key)
    )
    |> Multi.run(:remove_from_storage, fn _, _ ->
      case Reflection.remove_image_from_storage(record, container_name, old_image) do
        :ok ->
          {:ok, record}

        {:error, reason} ->
          Logger.error("Could not remove old image #{inspect(reason)}")

          changeset =
            Changeset.add_error(changeset, image_key, gettext("Could not remove old image"))

          {:error, changeset}
      end
    end)
    |> repo_module.transaction()
    |> case do
      {:ok, %{update_schema: schema}} ->
        {:ok, schema}

      {:error, _, %Changeset{} = changeset, _} ->
        {:error, changeset}
    end
  end
end
