defmodule Eakins.UseCases.DeleteSchemaImages do
  @moduledoc """
  Deletes several images from a schema.
  """
  alias Eakins.UseCases.Reflection
  alias Eakins.Repo

  require Logger

  def call(schema_module, id, container_name, image_keys) when is_binary(schema_module) do
    ("Elixir." <> schema_module)
    |> String.to_existing_atom()
    |> call(id, container_name, image_keys)
  end

  def call(schema_module, id, container_name, image_keys) do
    record = schema_module.record(id)
    changeset = schema_module.changeset(record, %{})

    Enum.reduce(image_keys, changeset, fn key, changeset ->
      if Reflection.has_image?(record, container_name, key) do
        Reflection.delete_image(record, container_name, changeset, key)
      else
        changeset
      end
    end)
    |> Repo.update()
  end
end
