defmodule Eakins.UseCases.Reflection.List do
  @moduledoc """
  Reflection utilities for image lists

  """
  alias Ecto.Changeset
  use Eakins.UseCases.Reflection

  @impl true
  def add_or_update_image(%schema_module{} = schema, container_name, image_key, uploaded_image) do
    changeset = schema_module.changeset(schema, %{})

    case find_image(schema, container_name, image_key) do
      nil ->
        add_name = fn_name("add", container_name)
        apply(schema_module, add_name, [changeset, uploaded_image])

      _image ->
        replace_name = fn_name("replace", container_name, "at")
        apply(schema_module, replace_name, [changeset, image_key, uploaded_image])
    end
  end

  @impl true
  def find_image(%schema_module{} = schema, container_name, image_key) do
    find_fn_name = fn_name(nil, container_name, "at")
    apply(schema_module, find_fn_name, [schema, image_key])
  end

  @impl true
  def delete_image(_, _, %Changeset{} = changeset, nil) do
    changeset
  end

  @impl true
  def delete_image(%schema_module{}, container_name, changeset, image_key) do
    delete_fn_name = fn_name("delete", container_name, "at")
    apply(schema_module, delete_fn_name, [changeset, image_key])
  end

  @impl true
  def remove_image_from_storage(%schema_module{} = schema, container_name, old_image) do
    remove_fn_name = fn_name("remove", container_name, "from_storage")
    apply(schema_module, remove_fn_name, [schema, old_image])
  end

  @impl true
  def update_image(%schema_module{}, container_name, changeset, updated_image) do
    update_fn_name = fn_name("replace", container_name, "at")
    apply(schema_module, update_fn_name, [changeset, updated_image.key, updated_image])
  end

  @impl true
  def has_image?(schema, container_name, image_key) do
    case find_image(schema, container_name, image_key) do
      nil -> false
      %{default: true} -> false
      _ -> true
    end
  end
end
