defmodule Eakins.UseCases.Reflection.Map do
  @moduledoc """
  Reflection utilities for image maps
  """
  use Eakins.UseCases.Reflection

  @impl true
  def add_or_update_image(%schema_module{} = schema, container_name, image_key, uploaded_image) do
    changeset = schema_module.changeset(schema, %{})
    put_image(schema, container_name, changeset, image_key, uploaded_image)
  end

  @impl true
  def find_image(%schema_module{} = schema, container_name, image_key) do
    find_fn_name = fn_name("find", container_name)
    apply(schema_module, find_fn_name, [schema, image_key])
  end

  @impl true
  def delete_image(_, _, changeset, nil) do
    changeset
  end

  @impl true
  def delete_image(%schema_module{}, container_name, changeset, image_key) do
    remove_fn_name = fn_name("delete", container_name)
    apply(schema_module, remove_fn_name, [changeset, image_key])
  end

  @impl true
  def remove_image_from_storage(%schema_module{} = schema, container_name, old_image) do
    remove_fn_name = fn_name("remove", container_name, "from_storage")
    apply(schema_module, remove_fn_name, [schema, old_image])
  end

  @impl true
  def update_image(%schema_module{}, container_name, changeset, updated_image) do
    put_fn_name = fn_name("put", container_name)
    apply(schema_module, put_fn_name, [changeset, updated_image])
  end

  @impl true
  def has_image?(%schema_module{} = schema, container_name, image_key) do
    singular_name = Inflex.singularize(container_name)
    has_fn_name = String.to_existing_atom("has_#{singular_name}?")
    apply(schema_module, has_fn_name, [schema, image_key])
  end

  defp put_image(%schema_module{}, container_name, changeset, key, uploaded_image) do
    put_fn_name = fn_name("put", container_name)
    apply(schema_module, put_fn_name, [changeset, key, uploaded_image])
  end
end
